/*
 *  Copyright (c) 2005 Boudewijn Rempt <boud@valdyas.org>
 *
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2, as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifndef _KO_TOOLBOX_PALETTE_
#define _KO_TOOLBOX_PALETTE_

#include <tqdockwindow.h>
#include <tqtoolbox.h>

#include <kopalettemanager.h>

/**
 * A palette based on a toolbox widget. This does not support drag and drop
 * configuration of palette widgets
 */
class KoToolBoxPalette : public KoPalette {

TQ_OBJECT
  

public:

    KoToolBoxPalette(TQWidget * parent, const char * name);
    ~KoToolBoxPalette();

public:
    void resetFont();
    virtual void plug(TQWidget * widget, const TQString & name, int position = INT_MAX);
    virtual void unplug(const TQWidget * widget);
    void showPage(TQWidget *w);
    void makeVisible(bool v);
    virtual void hidePage(TQWidget * w);
    int indexOf(TQWidget *w);
    bool isHidden(TQWidget *w);
    void togglePageHidden(TQWidget *w);
private:

    TQToolBox * m_page;

    TQMap<TQWidget*, int> m_hiddenPages;
};

#endif //_KO_TOOLBOX_PALETTE_
